/* solaris_time.c */
/* solaris only file */

#include <sys/time.h>
#include <signal.h>
#include <errno.h>

#ifndef TICKS_SHIFT
#define TICKS_SHIFT 0
#endif

extern unsigned long node_cputime;
extern int timed_out;

static struct timeval last_time;
static struct timeval first_time;

#define PARANOID
#ifdef PARANOID
static unsigned long last_cputime;
#endif

void MicroAlarmSigFun()
{
	timed_out = 1;
}

struct sigaction alarmaction;
sigset_t alarmblock, alarmoblock;

SunMicroTimeInit ()
{
	/* initialize sigaction for alarms */
	alarmaction.sa_handler = MicroAlarmSigFun;
	sigemptyset(&alarmaction.sa_mask );
	sigaddset(&alarmaction.sa_mask, SIGIO );
	alarmaction.sa_flags = SA_ONSTACK;
	sigaction ( SIGALRM, &alarmaction, (struct sigaction *) 0 );

	sigemptyset ( & alarmblock );
	sigemptyset ( & alarmoblock );
	sigaddset( &alarmblock, SIGALRM );
	sigaddset( &alarmblock, SIGIO );

	/* start clock */
	gettimeofday ( &last_time, 0 );
	first_time = last_time;
	node_cputime = 0;

#ifdef PARANOID
	last_cputime = node_cputime;
#endif
}

unsigned long MicroTime ()
{
	register long time, delta;
	struct timeval this_time;

	gettimeofday ( &this_time, 0 );

	time = this_time.tv_sec - last_time.tv_sec;
	delta = this_time.tv_usec - last_time.tv_usec;

	if ( time )
	{
		delta += 1000000 * time;
	}


	delta >>= TICKS_SHIFT;

	node_cputime += delta;

#ifdef PARANOID
	if ( node_cputime < last_cputime )
	{
		printf ( "node_cputime (%u)  last_cputime (%u) wrap around\n",
			node_cputime, last_cputime );
		printf ( "this_time (%d) secs (%d) usecs\n",
			this_time.tv_sec, this_time.tv_usec );
		printf ( "last_time (%d) secs (%d) usecs\n",
			last_time.tv_sec, last_time.tv_usec );
		printf ( "first_time (%d) secs (%d) usecs\n",
			first_time.tv_sec, first_time.tv_usec );
		printf ( "time (%d) delta (%d)\n", time, delta );
		tester();
	}

	last_cputime = node_cputime;

	if ( delta > 100000000 )
	{
		printf ( "delta > 100 second\n" );
		tester();
	}
#endif
	last_time = this_time;

	return delta;
}

#define DEFAULT_SECONDS 365 * 86400 /* seconds in a year */
#define DEFAULT_MICROSECONDS 0


SetMicroAlarm ( ms )
int ms;
{
	struct itimerval alarm_timer;
/*
	if (sigprocmask(SIG_BLOCK, &alarmblock, &alarmoblock) < 0)
		perror("sigprocmask");
*/
	alarm_timer.it_interval.tv_sec = DEFAULT_SECONDS;
	alarm_timer.it_interval.tv_usec = DEFAULT_MICROSECONDS;

	alarm_timer.it_value.tv_sec = ( ms / 1000 );
	alarm_timer.it_value.tv_usec = 1000 * ( ms % 1000 );
	/* setitimer bombs if tv_usec is >= 1,000,00 */

	if  ( setitimer ( ITIMER_REAL, &alarm_timer, 0 ) )
	{
		perror ("setitimer");
	}
/*
	sigprocmask(SIG_SETMASK, &alarmoblock, (sigset_t *)NULL);
*/
}

/* sort of stolen from itimer.c in the simulator */
/*
 * The difference of UserDeltaTime over the MicroTime:
 * This measures time in this process only, not wall clock time.
 */

static struct itimerval present_interval;    /* value read */
static struct itimerval new_interval;   /* new value set and old value */

unsigned long UserDeltaTime ()
{
    unsigned long delta, time;

    new_interval.it_value.tv_sec = DEFAULT_SECONDS;
    new_interval.it_value.tv_usec = 999000L;
    new_interval.it_interval.tv_sec = DEFAULT_SECONDS;
    new_interval.it_interval.tv_usec = 999000L;

	if ( setitimer(ITIMER_VIRTUAL, &new_interval, &present_interval) != 0)
		printf("setitimer failed\n");

    delta = new_interval.it_value.tv_usec - present_interval.it_value.tv_usec;
    time = new_interval.it_value.tv_sec - present_interval.it_value.tv_sec;

	if ( time )
	{
		delta += 1000000 * time;
	}

	delta >>= TICKS_SHIFT;

    return delta;
}

